#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

#define F_CPU 1000000


/*************************
Define pt 4 butoane
To Do: scrie cele 4 butoane aici
*************************/
#define buton_stanga PB0
#define buton_dreapta PB1
#define buton_sus PB2
#define buton_jos PB3
#define poz_linia_1 2 // int pentru pozitia liniei 1
#define poz_linia_2 14
#define poz_linia_3 26
#define DIM_LINE 7
#define SPACE_DIM 3 // 2 in realitate
/*************************
Matricea vectorizata pentru joc
Relevanta pe pozitiile 1-9
*************************/
int matrice[10];
int poz; //pozitie
int jucator;// 1 sau 10
char linia1[DIM_LINE];
char linia2[DIM_LINE];
char linia3[DIM_LINE];

/*************************
Functii pentru joc
*************************/

//seteaza stringurile ce vor fi afisate pe ecran
void update_string() {
	int i, line_nr, line_poz;
	for (i = 1; i < 10; i++) {
		line_nr = (i / 3) + 1;
		line_poz = (i % 3) * SPACE_DIM;
		if(line_nr == 1) {
			if(matrice[i] == 0)
				linia1[line_poz] = '_';
			if(matrice[i] == 1)
				linia1[line_poz] = 'X';
			if(matrice[i] == 10)
				linia1[line_poz] = 'O';
		} else if(line_nr == 2) {
			if(matrice[i] == 0)
				linia2[line_poz] = '_';
			if(matrice[i] == 1)
				linia2[line_poz] = 'X';
			if(matrice[i] == 10)
				linia2[line_poz] = 'O';
		} else {
			if(matrice[i] == 0)
				linia3[line_poz] = '_';
			if(matrice[i] == 1)
				linia3[line_poz] = 'X';
			if(matrice[i] == 10)
				linia3[line_poz] = 'O';	
		}
	}
	//afisez selectorul
	line_nr = (poz / 3) + 1;
	line_poz = (poz % 3) * SPACE_DIM;
	if(line_nr == 1) {
		linia1[line_poz] = 'S';
	} else if(line_nr == 2) {
		linia2[line_poz] = 'S';
	} else {
		linia3[line_poz] = 'S';
	}
}


//butonul de sus = start
void init_joc() {
	int i;
	for(i = 1; i < 10; i++)
		matrice[i] = 0; // 0 pt o casuta goala
	poz = 1;
	jucator = 1; // primul jucator "X"

	//initializez liniile care vor fi afisate
	for(i = 0; i < DIM_LINE; i++) {
		linia1[i] = ' '; //umplu cu spatiu
		linia2[i] = ' ';
		linia3[i] = ' ';

	}
	update_string();
}



//butonul din stanga
void move_left() {
	if(poz > 1) {
		poz--;
		update_string();
	}
}

//butonul din dreapta
void move_right() {
	if(poz < 9) {
		poz++;
		update_string();
	}
}

//se apeleaza in select
int verifica_joc_terminat() {
	int sum;
	//pe linii
	sum = matrice[1] + matrice[2] + matrice[3];
	if(sum == 3 || sum == 30)
		return 1;
	sum = matrice[4] + matrice[5] + matrice[6];
	if(sum == 3 || sum == 30)
		return 1;
	sum = matrice[7] + matrice[8] + matrice[9];
	if(sum == 3 || sum == 30)
		return 1;

	//pe coloane
	sum = matrice[1] + matrice[4] + matrice[7];
	if(sum == 3 || sum == 30)
		return 1;
	sum = matrice[2] + matrice[5] + matrice[8];
	if(sum == 3 || sum == 30)
		return 1;
	sum = matrice[3] + matrice[6] + matrice[9];
	if(sum == 3 || sum == 30)
		return 1;

	//pe diagonale
	sum = matrice[1] + matrice[5] + matrice[9];
	if(sum == 3 || sum == 30)
		return 1;
	sum = matrice[3] + matrice[5] + matrice[7];
	if(sum == 3 || sum == 30)
		return 1;

	return 0; //still to play
}

//se apeleaza la select
void schimba_jucatorul() {
	if(jucator == 1)
		jucator = 10;
	else
		jucator = 1;
}

//se apeleaza pt butonul de jos
int select() {
	if(matrice[poz] == 0) {
		matrice[poz] = jucator;
		schimba_jucatorul();
		update_string();
		if( verifica_joc_terminat() ) {
			init_joc();
		}
	}
}


/*************************
Functii pentru butoane si pentru afisare
*************************/
void init_buttons() {
	//fac init pe cele 4 butoane
	DDRD &= ~(1 << buton_jos);
	PORTD |= (1 << buton_jos);

	DDRD &= ~(1 << buton_sus);
	PORTD |= (1 << buton_sus);

	DDRD &= ~(1 << buton_dreapta);
	PORTD |= (1 << buton_dreapta);

	DDRD &= ~(1 << buton_stanga);
	PORTD |= (1 << buton_stanga);
}

void print_game() {
	lcd_clear();
	lcd_update();
	
	lcd_gotoXY(poz_linia_1, 0);
	lcd_str(linia1, DIM_LINE);

	lcd_gotoXY(poz_linia_2, 0);
	lcd_str(linia2, DIM_LINE);

	lcd_gotoXY(poz_linia_3, 0);
	lcd_str(linia3, DIM_LINE);
}

void Delay(void) {

	volatile int i;
	for (i = -1000000; i < 100000; i++);

}

void play_game() {
	init_buttons();
	init_joc();
	while(1) {
	//verific care buton e apasat
	//apelez functia corespunzatoare butonului
		
		//resetez jocul
		if( (PIND & (1 << buton_sus)) == 0) {
			init_joc();
			print_game();
			Delay();
		}

		//selectez casuta curenta
		if( (PIND & (1 << buton_jos)) == 0) {
			select();
			print_game();
			Delay();
		}

		//mut selectorul la stanga
		if( (PIND & (1 << buton_stanga)) == 0) {
			move_left();
			print_game();
			Delay();
		}

		//mut selectorul la dreapta
		if( (PIND & (1 << buton_dreapta)) == 0) {
			move_right();
			print_game();
			Delay();
		}

	//apoi apelez functia de afisare pentru cele 3 linii
	//apoi Delay sau poate sleep
	}
}








void print_name(void) {

	lcd_clear();
	lcd_update();
	
	lcd_gotoXY(2, 0);
	lcd_str("Mihai Bleont", 12);
}

void print_mode(void) {
	
	print_name();
	
	while(1) {
		print_name();
		Delay(); 
		
	}
}

int main(void) {
	
	lcd_init();
	lcd_clear();
	lcd_update();
	
	//print_mode();
	play_game();
}